# -*- coding: utf-8 -*-
"""
Django settings for test_site project.

Generated by 'django-admin startproject' using Django 2.0.5.

For more information on this file, see
https://docs.djangoproject.com/en/2.0/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.0/ref/settings/
"""

import os


# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/2.0/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "6p=(!%c2i2o2chq9o!qgks&09-d67k^!l&m(&g$==2va#me)mh"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ["*"]


# Application definition

INSTALLED_APPS = [
    "test_site",
    "django_extensions",
    "django_sorcery",
    "polls.apps.PollsConfig",
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
]

MIDDLEWARE = [
    "django_sorcery.db.middleware.SQLAlchemyMiddleware",
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

ROOT_URLCONF = "test_site.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    }
]

WSGI_APPLICATION = "test_site.wsgi.application"


# Database
# https://docs.djangoproject.com/en/2.0/ref/settings/#databases

DATABASES = {"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": os.path.join(BASE_DIR, "db.sqlite3")}}


# Password validation
# https://docs.djangoproject.com/en/2.0/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {"NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator"},
    {"NAME": "django.contrib.auth.password_validation.MinimumLengthValidator"},
    {"NAME": "django.contrib.auth.password_validation.CommonPasswordValidator"},
    {"NAME": "django.contrib.auth.password_validation.NumericPasswordValidator"},
]


# Internationalization
# https://docs.djangoproject.com/en/2.0/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.0/howto/static-files/

STATIC_URL = "/static/"



# -*- coding: utf-8 -*-

from sqlalchemy.orm import configure_mappers

from django.apps import AppConfig


class PollsConfig(AppConfig):
    name = "polls"

    def ready(self):
        configure_mappers()



# -*- coding: utf-8 -*-
"""
Testing utilities
"""

from sqlalchemy import event
from sqlalchemy.engine import Engine

from .db import databases


class CommitException(BaseException):
    """
    Raised when a commit happens during testing
    """


class Transact(object):
    """
    Helper context manager for handling transactions during tests. Perfect for testing a single
    unit of work.

    For testing sqlalchemy apps the common pattern is to usually, start a transaction or savepoint,
    run the tests and once the test is done rollback. Additionally it also prevents commits
    to the database so that every test gets the same database state.
    """

    @staticmethod
    def _commit(connection, *args, **kwargs):
        raise CommitException("Commits are not allowed")

    def start(self):
        """
        Starts transaction management and hooks commit events
        """
        self.hook()

    def stop(self):
        """
        Stops transaction management, rolls back transactions and unhooks commit events
        """
        self.end()
        self.unhook()

    def hook(self):
        """
        Hooks commit events to prevent commits
        """
        if not event.contains(Engine, "commit", self._commit):
            event.listen(Engine, "commit", self._commit)
        if not event.contains(Engine, "commit_twophase", self._commit):
            event.listen(Engine, "commit_twophase", self._commit)

    def unhook(self):
        """
        Unhooks commit events
        """
        if event.contains(Engine, "commit", self._commit):
            event.remove(Engine, "commit", self._commit)
        if event.contains(Engine, "commit_twophase", self._commit):
            event.remove(Engine, "commit_twophase", self._commit)

    def end(self):
        """
        Rolls back transaction and removes session
        """
        databases.rollback()
        databases.remove()

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.stop()



# -*- coding: utf-8 -*-
"""
pytest plugins
"""

import pytest

from .db.profiler import SQLAlchemyProfiler
from .testing import Transact


@pytest.fixture(scope="function")
def sqlalchemy_profiler():
    """
    pytest fixture for sqlalchemy profiler
    """
    with SQLAlchemyProfiler() as profiler:
        yield profiler


@pytest.fixture(scope="function")
def transact():
    """
    pytest transact fixture for sqlalchemy
    """
    with Transact() as t:
        yield t



# -*- coding: utf-8 -*-
"""
Default alembic signal configuration
"""

from ...db import signals


def include_object(obj, name, type_, reflected, compare_to):
    """
    The default include_object handler for alembic, delegates to alembic_include_object signal

    The return value will be considered True if all the signal handlers return True for the
    object to be included in migration.
    """
    results = signals.alembic_include_object.send(
        obj, name=name, type_=type_, reflected=reflected, compare_to=compare_to
    )
    if results:
        return all(res[1] for res in results)

    return True


def process_revision_directives(context, revision, directives):
    """
    The default process_revision_directives handler for alembic, delegates to
    alembic_process_revision_directives
    """
    signals.alembic_process_revision_directives.send(context, revision=revision, directives=directives)
